@@ -39,7 +39,6 @@ typedef struct {
   gboolean wp_update_gconf;
   GtkIconView *wp_view;
   GtkTreeModel *wp_model;
-  GtkCellRenderer *wp_cell;
   GtkWidget *wp_scpicker;
   GtkWidget *wp_pcpicker;
   GtkWidget *wp_style_menu;
@@ -47,6 +46,7 @@ typedef struct {
   GtkWidget *wp_rem_button;
   GtkWidget *wp_filesel;
   GtkWidget *wp_image;
+  GSList *wp_uris;
 
   /* font */
   GtkWidget *font_details;
@@ -56,6 +56,10 @@ typedef struct {
   GnomeThemeMetaInfo *theme_custom;
   GdkPixbuf *theme_icon;
   GtkWidget *theme_save_dialog;
+  GtkWidget *theme_message_area;
+  GtkWidget *theme_message_label;
+  GtkWidget *apply_background_button;
+  GtkWidget *apply_font_button;
 
   /* style */
   GdkPixbuf *gtk_theme_icon;
